import { Location, Node, NodeEntry, Operation, Path, PathRef, Point, PointRef, Range, RangeRef, Span, Text, AncestorOf, ElementOf, NodeOf, MarksOf, TextOf, Element, NodeMatch } from '..';
export declare type Value = Element[];
/**
 * The `Editor` interface stores all the state of a Slate editor. It is extended
 * by plugins that wish to add their own helpers and implement new behaviors.
 */
export declare type Editor<V extends Value> = {
    children: V;
    selection: Selection;
    operations: Operation[];
    marks: Record<string, any> | null;
    [key: string]: unknown;
    isInline: (element: Element) => boolean;
    isVoid: (element: Element) => boolean;
    normalizeNode: (entry: NodeEntry) => void;
    onChange: () => void;
    addMark: (key: string, value: any) => void;
    apply: (operation: Operation) => void;
    deleteBackward: (unit: 'character' | 'word' | 'line' | 'block') => void;
    deleteForward: (unit: 'character' | 'word' | 'line' | 'block') => void;
    deleteFragment: (direction?: 'forward' | 'backward') => void;
    getFragment: () => Array<Element | Text>;
    insertBreak: () => void;
    insertFragment: (fragment: Array<Element | Text>) => void;
    insertNode: (node: Element | Text | Array<Element | Text>) => void;
    insertText: (text: string) => void;
    removeMark: (key: string) => void;
};
export declare const Editor: {
    /**
     * Get the ancestor above a location in the document.
     */
    above<V extends Value>(editor: Editor<V>, options?: {
        at?: Location | undefined;
        match?: NodeMatch<AncestorOf<Editor<V>>> | undefined;
        mode?: "highest" | "lowest" | undefined;
        voids?: boolean | undefined;
    }): NodeEntry<AncestorOf<Editor<V>>> | undefined;
    /**
     * Add a custom property to the leaf text nodes in the current selection.
     *
     * If the selection is currently collapsed, the marks will be added to the
     * `editor.marks` property instead, and applied when text is inserted next.
     */
    addMark<V_1 extends Value, M extends import("../utils/types").Simplify<import("../utils/types").UnionToIntersection<import("./node").NodeProps<TextOf<Editor<V_1>>>>>, K extends keyof M & string>(editor: Editor<V_1>, key: {} extends M ? string : K, value: {} extends M ? unknown : M[K]): void;
    /**
     * Get the point after a location.
     */
    after<V_2 extends Value>(editor: Editor<V_2>, at: Location, options?: {
        distance?: number;
        unit?: 'offset' | 'character' | 'word' | 'line' | 'block';
        voids?: boolean;
    }): Point | undefined;
    /**
     * Get the point before a location.
     */
    before<V_3 extends Value>(editor: Editor<V_3>, at: Location, options?: {
        distance?: number;
        unit?: 'offset' | 'character' | 'word' | 'line' | 'block';
        voids?: boolean;
    }): Point | undefined;
    /**
     * Delete content in the editor backward from the current selection.
     */
    deleteBackward<V_4 extends Value>(editor: Editor<V_4>, options?: {
        unit?: 'character' | 'word' | 'line' | 'block';
    }): void;
    /**
     * Delete content in the editor forward from the current selection.
     */
    deleteForward<V_5 extends Value>(editor: Editor<V_5>, options?: {
        unit?: 'character' | 'word' | 'line' | 'block';
    }): void;
    /**
     * Delete the content in the current selection.
     */
    deleteFragment<V_6 extends Value>(editor: Editor<V_6>, options?: {
        direction?: 'forward' | 'backward';
    }): void;
    /**
     * Get the start and end points of a location.
     */
    edges<V_7 extends Value>(editor: Editor<V_7>, at: Location): [Point, Point];
    /**
     * Get the end point of a location.
     */
    end<V_8 extends Value>(editor: Editor<V_8>, at: Location): Point;
    /**
     * Get the first node at a location.
     */
    first<V_9 extends Value>(editor: Editor<V_9>, at: Location): NodeEntry<NodeOf<Editor<V_9>>>;
    /**
     * Get the fragment at a location.
     */
    fragment<V_10 extends Value>(editor: Editor<V_10>, at: Location): (ElementOf<Editor<V_10>> | TextOf<Editor<V_10>>)[];
    /**
     * Check if a node has block children.
     */
    hasBlocks<V_11 extends Value>(editor: Editor<V_11>, element: Element): boolean;
    /**
     * Check if a node has inline and text children.
     */
    hasInlines<V_12 extends Value>(editor: Editor<V_12>, element: Element): boolean;
    /**
     * Check if a node has text children.
     */
    hasTexts<V_13 extends Value>(editor: Editor<V_13>, element: Element): boolean;
    /**
     * Insert a block break at the current selection.
     *
     * If the selection is currently expanded, it will be deleted first.
     */
    insertBreak<V_14 extends Value>(editor: Editor<V_14>): void;
    /**
     * Insert a fragment at the current selection.
     *
     * If the selection is currently expanded, it will be deleted first.
     */
    insertFragment<V_15 extends Value>(editor: Editor<V_15>, fragment: (ElementOf<Editor<V_15>> | TextOf<Editor<V_15>>)[]): void;
    /**
     * Insert a node at the current selection.
     *
     * If the selection is currently expanded, it will be deleted first.
     */
    insertNode<V_16 extends Value>(editor: Editor<V_16>, node: ElementOf<Editor<V_16>> | TextOf<Editor<V_16>> | (ElementOf<Editor<V_16>> | TextOf<Editor<V_16>>)[]): void;
    /**
     * Insert text at the current selection.
     *
     * If the selection is currently expanded, it will be deleted first.
     */
    insertText<V_17 extends Value>(editor: Editor<V_17>, text: string): void;
    /**
     * Check if a value is a block `Element` object.
     */
    isBlock<V_18 extends Value>(editor: Editor<V_18>, value: any): value is Element;
    /**
     * Check if a value is an `Editor` object.
     */
    isEditor(value: any): value is Editor<Value>;
    /**
     * Check if a point is the end point of a location.
     */
    isEnd<V_19 extends Value>(editor: Editor<V_19>, point: Point, at: Location): boolean;
    /**
     * Check if a point is an edge of a location.
     */
    isEdge<V_20 extends Value>(editor: Editor<V_20>, point: Point, at: Location): boolean;
    /**
     * Check if an element is empty, accounting for void nodes.
     */
    isEmpty<V_21 extends Value>(editor: Editor<V_21>, element: ElementOf<Editor<V_21>>): boolean;
    /**
     * Check if a value is an inline `Element` object.
     */
    isInline<V_22 extends Value>(editor: Editor<V_22>, value: any): value is Element;
    /**
     * Check if the editor is currently normalizing after each operation.
     */
    isNormalizing<V_23 extends Value>(editor: Editor<V_23>): boolean;
    /**
     * Check if a point is the start point of a location.
     */
    isStart<V_24 extends Value>(editor: Editor<V_24>, point: Point, at: Location): boolean;
    /**
     * Check if a value is a void `Element` object.
     */
    isVoid<V_25 extends Value>(editor: Editor<V_25>, value: any): value is Element;
    /**
     * Get the last node at a location.
     */
    last<V_26 extends Value>(editor: Editor<V_26>, at: Location): NodeEntry<NodeOf<Editor<V_26>>>;
    /**
     * Get the leaf text node at a location.
     */
    leaf<V_27 extends Value>(editor: Editor<V_27>, at: Location, options?: {
        depth?: number;
        edge?: 'start' | 'end';
    }): NodeEntry<TextOf<Editor<V_27>>>;
    /**
     * Iterate through all of the levels at a location.
     */
    levels<V_28 extends Value, N extends Node>(editor: Editor<V_28>, options?: {
        at?: Location | undefined;
        match?: NodeMatch<N & NodeOf<Editor<V_28>>> | undefined;
        reverse?: boolean | undefined;
        voids?: boolean | undefined;
    }): Generator<NodeEntry<N & NodeOf<Editor<V_28>>>, void, undefined>;
    /**
     * Get the marks that would be added to text at the current selection.
     */
    marks<V_29 extends Value>(editor: Editor<V_29>): Partial<import("../utils/types").Simplify<import("../utils/types").UnionToIntersection<import("./node").NodeProps<TextOf<Editor<V_29>>>>>> | null;
    /**
     * Get the matching node in the branch of the document after a location.
     */
    next<V_30 extends Value, N_1 extends Node>(editor: Editor<V_30>, options?: {
        at?: Location | undefined;
        match?: NodeMatch<N_1 & NodeOf<Editor<V_30>>> | undefined;
        mode?: "highest" | "lowest" | "all" | undefined;
        voids?: boolean | undefined;
    }): NodeEntry<N_1 & NodeOf<Editor<V_30>>> | undefined;
    /**
     * Get the node at a location.
     */
    node<V_31 extends Value>(editor: Editor<V_31>, at: Location, options?: {
        depth?: number;
        edge?: 'start' | 'end';
    }): NodeEntry<NodeOf<Editor<V_31>>>;
    /**
     * Iterate through all of the nodes in the Editor.
     */
    nodes<V_32 extends Value, T extends Node>(editor: Editor<V_32>, options?: {
        at?: Location | Span | undefined;
        match?: NodeMatch<T & NodeOf<Editor<V_32>>> | undefined;
        mode?: "highest" | "lowest" | "all" | undefined;
        universal?: boolean | undefined;
        reverse?: boolean | undefined;
        voids?: boolean | undefined;
    }): Generator<NodeEntry<T & NodeOf<Editor<V_32>>>, void, undefined>;
    /**
     * Normalize any dirty objects in the editor.
     */
    normalize<V_33 extends Value>(editor: Editor<V_33>, options?: {
        force?: boolean;
    }): void;
    /**
     * Get the parent node of a location.
     */
    parent<V_34 extends Value>(editor: Editor<V_34>, at: Location, options?: {
        depth?: number;
        edge?: 'start' | 'end';
    }): NodeEntry<AncestorOf<Editor<V_34>>>;
    /**
     * Get the path of a location.
     */
    path<V_35 extends Value>(editor: Editor<V_35>, at: Location, options?: {
        depth?: number;
        edge?: 'start' | 'end';
    }): Path;
    /**
     * Create a mutable ref for a `Path` object, which will stay in sync as new
     * operations are applied to the editor.
     */
    pathRef<V_36 extends Value>(editor: Editor<V_36>, path: Path, options?: {
        affinity?: 'backward' | 'forward' | null;
    }): PathRef;
    /**
     * Get the set of currently tracked path refs of the editor.
     */
    pathRefs<V_37 extends Value>(editor: Editor<V_37>): Set<PathRef>;
    /**
     * Get the start or end point of a location.
     */
    point<V_38 extends Value>(editor: Editor<V_38>, at: Location, options?: {
        edge?: 'start' | 'end';
    }): Point;
    /**
     * Create a mutable ref for a `Point` object, which will stay in sync as new
     * operations are applied to the editor.
     */
    pointRef<V_39 extends Value>(editor: Editor<V_39>, point: Point, options?: {
        affinity?: 'backward' | 'forward' | null;
    }): PointRef;
    /**
     * Get the set of currently tracked point refs of the editor.
     */
    pointRefs<V_40 extends Value>(editor: Editor<V_40>): Set<PointRef>;
    /**
     * Iterate through all of the positions in the document where a `Point` can be
     * placed.
     *
     * By default it will move forward by individual offsets at a time,  but you
     * can pass the `unit: 'character'` option to moved forward one character, word,
     * or line at at time.
     *
     * Note: By default void nodes are treated as a single point and iteration
     * will not happen inside their content unless you pass in true for the
     * voids option, then iteration will occur.
     */
    positions<V_41 extends Value>(editor: Editor<V_41>, options?: {
        at?: Location;
        unit?: 'offset' | 'character' | 'word' | 'line' | 'block';
        reverse?: boolean;
        voids?: boolean;
    }): Generator<Point, void, undefined>;
    /**
     * Get the matching node in the branch of the document before a location.
     */
    previous<V_42 extends Value, T_1 extends Node>(editor: Editor<V_42>, options?: {
        at?: Location | undefined;
        match?: NodeMatch<T_1 & NodeOf<Editor<V_42>>> | undefined;
        mode?: "highest" | "lowest" | "all" | undefined;
        voids?: boolean | undefined;
    }): NodeEntry<T_1 & NodeOf<Editor<V_42>>> | undefined;
    /**
     * Get a range of a location.
     */
    range<V_43 extends Value>(editor: Editor<V_43>, at: Location, to?: Location | undefined): Range;
    /**
     * Create a mutable ref for a `Range` object, which will stay in sync as new
     * operations are applied to the editor.
     */
    rangeRef<V_44 extends Value>(editor: Editor<V_44>, range: Range, options?: {
        affinity?: 'backward' | 'forward' | 'outward' | 'inward' | null;
    }): RangeRef;
    /**
     * Get the set of currently tracked range refs of the editor.
     */
    rangeRefs<V_45 extends Value>(editor: Editor<V_45>): Set<RangeRef>;
    /**
     * Remove a custom property from all of the leaf text nodes in the current
     * selection.
     *
     * If the selection is currently collapsed, the removal will be stored on
     * `editor.marks` and applied to the text inserted next.
     */
    removeMark<V_46 extends Value, M_1 extends import("../utils/types").Simplify<import("../utils/types").UnionToIntersection<import("./node").NodeProps<TextOf<Editor<V_46>>>>>>(editor: Editor<V_46>, key: {} extends M_1 ? string : keyof M_1 & string): void;
    /**
     * Get the start point of a location.
     */
    start<V_47 extends Value>(editor: Editor<V_47>, at: Location): Point;
    /**
     * Get the text string content of a location.
     *
     * Note: by default the text of void nodes is considered to be an empty
     * string, regardless of content, unless you pass in true for the voids option
     */
    string<V_48 extends Value>(editor: Editor<V_48>, at: Location, options?: {
        voids?: boolean;
    }): string;
    /**
     * Convert a range into a non-hanging one.
     */
    unhangRange<V_49 extends Value>(editor: Editor<V_49>, range: Range, options?: {
        voids?: boolean;
    }): Range;
    /**
     * Match a void node in the current branch of the editor.
     */
    void<V_50 extends Value>(editor: Editor<V_50>, options?: {
        at?: Location;
        mode?: 'highest' | 'lowest';
        voids?: boolean;
    }): NodeEntry<ElementOf<Editor<V_50>>> | undefined;
    /**
     * Call a function, deferring normalization until after it completes.
     */
    withoutNormalizing<V_51 extends Value>(editor: Editor<V_51>, fn: () => void): void;
};
/**
 * The `Selection` interface is a special type of `Range` that can contain extra
 * properties, and can be `null` to mean no selection exists in the editor.
 */
export declare type Selection = Range | null;
/**
 * A helper type for getting the value of an editor.
 */
export declare type ValueOf<E extends Editor<Value>> = E['children'];
//# sourceMappingURL=editor.d.ts.map