import { Editor, Path, Range, Text, Element, ElementOf, TextOf, Value } from '..';
/**
 * The `Node` union type represents all of the different types of nodes that
 * occur in a Slate document tree.
 */
export declare type Node = Editor<Value> | Element | Text;
export declare const Node: {
    /**
     * Get the node at a specific path, asserting that it's an ancestor node.
     */
    ancestor<N extends Node>(root: N, path: Path): AncestorOf<N>;
    /**
     * Return a generator of all the ancestor nodes above a specific path.
     *
     * By default the order is bottom-up, from lowest to highest ancestor in
     * the tree, but you can pass the `reverse: true` option to go top-down.
     */
    ancestors<N_1 extends Node>(root: N_1, path: Path, options?: {
        reverse?: boolean;
    }): Generator<NodeEntry<AncestorOf<N_1>>, void, undefined>;
    /**
     * Get the child of a node at a specific index.
     */
    child<N_2 extends Node, I extends number>(root: N_2, index: I): ChildOf<N_2, I>;
    /**
     * Iterate over the children of a node at a specific path.
     */
    children<N_3 extends Node>(root: N_3, path: Path, options?: {
        reverse?: boolean;
    }): Generator<NodeEntry<ChildOf<N_3, number>>, void, undefined>;
    /**
     * Get an entry for the common ancesetor node of two paths.
     */
    common<N_4 extends Node>(root: N_4, path: Path, another: Path): NodeEntry<NodeOf<N_4>>;
    /**
     * Get the node at a specific path, asserting that it's a descendant node.
     */
    descendant<N_5 extends Node>(root: N_5, path: Path): DescendantOf<N_5>;
    /**
     * Return a generator of all the descendant node entries inside a root node.
     */
    descendants<N_6 extends Node>(root: N_6, options?: {
        from?: Path | undefined;
        to?: Path | undefined;
        reverse?: boolean | undefined;
        pass?: ((node: NodeEntry<DescendantOf<N_6>>) => boolean) | undefined;
    }): Generator<NodeEntry<DescendantOf<N_6>>, void, undefined>;
    /**
     * Return a generator of all the element nodes inside a root node. Each iteration
     * will return an `ElementEntry` tuple consisting of `[Element, Path]`. If the
     * root node is an element it will be included in the iteration as well.
     */
    elements<N_7 extends Node>(root: N_7, options?: {
        from?: Path | undefined;
        to?: Path | undefined;
        reverse?: boolean | undefined;
        pass?: ((node: NodeEntry<ElementOf<N_7>>) => boolean) | undefined;
    }): Generator<NodeEntry<ElementOf<N_7>>, void, undefined>;
    /**
     * Get the first node entry in a root node from a path.
     */
    first<N_8 extends Node>(root: N_8, path: Path): NodeEntry<NodeOf<N_8>>;
    /**
     * Get the sliced fragment represented by a range inside a root node.
     */
    fragment<N_9 extends Node>(root: N_9, range: Range): (ElementOf<N_9> | TextOf<N_9>)[];
    /**
     * Get the descendant node referred to by a specific path. If the path is an
     * empty array, it refers to the root node itself.
     */
    get<N_10 extends Node>(root: N_10, path: Path): NodeOf<N_10>;
    /**
     * Check if a descendant node exists at a specific path.
     */
    has(root: Node, path: Path): boolean;
    /**
     * Check if a value implements the 'Ancestor' interface.
     */
    isAncestor(value: any): value is Ancestor;
    /**
     * Check if a value implements the `Node` interface.
     */
    isNode(value: any): value is Node;
    /**
     * Check if a value is a list of `Node` objects.
     */
    isNodeList(value: any): value is Node[];
    /**
     * Get the last node entry in a root node from a path.
     */
    last<N_11 extends Node>(root: N_11, path: Path): NodeEntry<NodeOf<N_11>>;
    /**
     * Get the node at a specific path, ensuring it's a leaf text node.
     */
    leaf<N_12 extends Node>(root: N_12, path: Path): TextOf<N_12>;
    /**
     * Return a generator of the in a branch of the tree, from a specific path.
     *
     * By default the order is top-down, from lowest to highest node in the tree,
     * but you can pass the `reverse: true` option to go bottom-up.
     */
    levels<N_13 extends Node>(root: N_13, path: Path, options?: {
        reverse?: boolean;
    }): Generator<NodeEntry<NodeOf<N_13>>, void, undefined>;
    /**
     * Check if a node matches a set of props.
     */
    matches(node: Node, props: object): boolean;
    /**
     * Return a generator of all the node entries of a root node. Each entry is
     * returned as a `[Node, Path]` tuple, with the path referring to the node's
     * position inside the root node.
     */
    nodes<N_14 extends Node>(root: N_14, options?: {
        from?: Path | undefined;
        to?: Path | undefined;
        reverse?: boolean | undefined;
        pass?: ((entry: NodeEntry<NodeOf<N_14>>) => boolean) | undefined;
    }): Generator<NodeEntry<NodeOf<N_14>>, void, undefined>;
    /**
     * Get the parent of a node at a specific path.
     */
    parent<N_15 extends Node>(root: N_15, path: Path): AncestorOf<N_15>;
    /**
     * Extract the custom properties from a node.
     */
    props<N_16 extends Node>(node: N_16): NodeProps<N_16>;
    /**
     * Get the concatenated text string of a node's content.
     *
     * Note that this will not include spaces or line breaks between block nodes.
     * It is not a user-facing string, but a string for performing offset-related
     * computations for a node.
     */
    string(node: Node): string;
    /**
     * Return a generator of all leaf text nodes in a root node.
     */
    texts<N_17 extends Node>(root: N_17, options?: {
        from?: Path | undefined;
        to?: Path | undefined;
        reverse?: boolean | undefined;
        pass?: ((node: NodeEntry<NodeOf<N_17>>) => boolean) | undefined;
    }): Generator<NodeEntry<TextOf<N_17>>, void, undefined>;
};
/**
 * `NodeEntry` objects are returned when iterating over the nodes in a Slate
 * document tree. They consist of the node and its `Path` relative to the root
 * node in the document.
 */
export declare type NodeEntry<N extends Node = Node> = [N, Path];
/**
 * A utility type to get all the node types from a root node type.
 */
export declare type NodeOf<N extends Node> = N | ElementOf<N> | TextOf<N>;
/**
 * Convenience type for returning the props of a node.
 */
export declare type NodeProps<N extends Node> = N extends Editor<Value> ? Omit<N, 'children'> : N extends Element ? Omit<N, 'children'> : Omit<N, 'text'>;
/**
 * A helper type for narrowing matched nodes with a predicate.
 */
export declare type NodeMatch<T extends Node> = ((node: Node, path: Path) => node is T) | ((node: Node, path: Path) => boolean);
/**
 * The `Ancestor` union type represents nodes that are ancestors in the tree.
 * It is returned as a convenience in certain cases to narrow a value further
 * than the more generic `Node` union.
 */
export declare type Ancestor = Editor<Value> | Element;
/**
 * A utility type to get all the ancestor node types from a root node type.
 */
export declare type AncestorOf<N extends Node> = Editor<Value> extends N ? Editor<Value> | Element : Element extends N ? Element : N extends Editor<Value> ? N | N['children'][number] | ElementOf<N['children'][number]> : N extends Element ? N | ElementOf<N> : never;
/**
 * The `Descendant` union type represents nodes that are descendants in the
 * tree. It is returned as a convenience in certain cases to narrow a value
 * further than the more generic `Node` union.
 */
export declare type Descendant = Element | Text;
/**
 * A utility type to get all the descendant node types from a root node type.
 */
export declare type DescendantOf<N extends Node> = N extends Editor<Value> ? ElementOf<N> | TextOf<N> : N extends Element ? ElementOf<N['children'][number]> | TextOf<N> : never;
/**
 * A utility type to get the child node types from a root node type.
 */
export declare type ChildOf<N extends Node, I extends number = number> = N extends Editor<Value> ? N['children'][I] : N extends Element ? N['children'][I] : never;
//# sourceMappingURL=node.d.ts.map